import { Overlay } from '@angular/cdk/overlay';
import { ComponentPortal } from '@angular/cdk/portal';
import { EventEmitter, Component, Input, Output, HostListener, Directive, ElementRef, forwardRef, ViewChild, Injectable, Pipe, HostBinding, NgModule } from '@angular/core';
import { FormControl, NG_VALUE_ACCESSOR, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { Subject, timer } from 'rxjs';
import { debounce } from 'rxjs/operators';
import * as MarkdownIt from 'markdown-it';
import * as TaskList from 'markdown-it-task-lists';
import { __awaiter } from 'tslib';
import { DomSanitizer } from '@angular/platform-browser';
import { TextFieldModule } from '@angular/cdk/text-field';
import { CommonModule } from '@angular/common';
import { MatButtonModule } from '@angular/material/button';
import { MatIconModule } from '@angular/material/icon';
import { MatListModule } from '@angular/material/list';

class EditorHistory {
    constructor(content, selectionStart, selectionEnd) {
        this.content = content;
        this.selectionStart = selectionStart;
        this.selectionEnd = selectionEnd;
    }
}

class NgSimpleMarkdownMentionsComponent {
    constructor() {
        this.count = new EventEmitter(true);
        this.destroy = new EventEmitter(true);
        this.selected = new EventEmitter(true);
        this.activeIndex = 0;
        this.items = [];
    }
    keyboardEvent(event) {
        // get key press
        const key = event.key.toLowerCase();
        // check action
        switch (key) {
            case 'enter': {
                if (this.items.length > 0) {
                    this.onClick(this.activeIndex);
                    // stop event
                    event.preventDefault();
                    event.stopPropagation();
                }
                break;
            }
            case 'escape': {
                this.destroy.emit();
                // stop event
                event.preventDefault();
                event.stopPropagation();
                break;
            }
            case 'arrowdown': {
                if (this.items.length > 0) {
                    // increase index
                    this.activeIndex++;
                    // check overflow
                    if (this.activeIndex > this.items.length - 1) {
                        this.activeIndex = 0;
                    }
                    // stop event
                    event.preventDefault();
                    event.stopPropagation();
                }
                break;
            }
            case 'arrowup': {
                if (this.items.length > 0) {
                    // increase index
                    this.activeIndex--;
                    // check overflow
                    if (this.activeIndex < 0) {
                        this.activeIndex = this.items.length - 1;
                    }
                    // stop event
                    event.preventDefault();
                    event.stopPropagation();
                }
                break;
            }
        }
    }
    ngOnInit() {
        this.subscription = this.data.subscribe((items) => {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            this.items = [...items];
            this.count.emit(this.items.length);
        });
    }
    ngOnDestroy() {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    }
    get hasResult() {
        return this.items && this.items.length > 0;
    }
    onClick(index) {
        this.selected.emit(this.items[index]);
    }
}
NgSimpleMarkdownMentionsComponent.decorators = [
    { type: Component, args: [{
                template: "<mat-nav-list dense class=\"ng-simple-markdown-editor-mentions\" *ngIf=\"hasResult\">\n\t<a\n\t\tmat-list-item\n\t\t*ngFor=\"let item of items; let i = index\"\n\t\tclass=\"ng-simple-markdown-editor-mentions__item\"\n\t\t[class.ng-simple-markdown-editor-mentions__item--active]=\"activeIndex === i\"\n\t\t(click)=\"onClick(i)\"\n\t>\n\t\t<ng-container *ngTemplateOutlet=\"template ? template : defaultTemplate; context: { item: item }\"></ng-container>\n\t\t<ng-template #defaultTemplate>{{item}}</ng-template>\n\t</a>\n</mat-nav-list>\n",
                styles: [".ng-simple-markdown-editor-mentions{background-color:#fff;border:1px solid #ccc}.ng-simple-markdown-editor-mentions__item{-moz-user-select:none;-ms-user-select:none;-webkit-user-select:none;user-select:none}.ng-simple-markdown-editor-mentions__item--active{background-color:#eee}"]
            },] }
];
NgSimpleMarkdownMentionsComponent.propDecorators = {
    data: [{ type: Input }],
    template: [{ type: Input }],
    count: [{ type: Output }],
    destroy: [{ type: Output }],
    selected: [{ type: Output }],
    keyboardEvent: [{ type: HostListener, args: ['document:keydown', ['$event'],] }]
};

class NgSimpleMarkdownMentionDirective {
    constructor(elementRef, overlay) {
        this.elementRef = elementRef;
        this.overlay = overlay;
        this.searchResult = new EventEmitter(true);
        this.searchTerm = new EventEmitter(true);
        this.menuOpened = false;
        this.itemCount = 0;
        this.triggerCharPosition = 0;
        this.triggerChar = '';
        this.triggerTerm = '';
        this.triggerMap = {};
        this.searchTermDebounce = new Subject();
    }
    onInput(event) {
        // check if any trigger
        if (!this.configs) {
            return;
        }
        // when typed text
        if (event.inputType === 'insertText' && event.data) {
            // get the last char
            const char = Array.from(event.data).pop();
            // exit when typed space
            if (char === ' ') {
                this.closeMenu();
                return;
            }
            // check if char in trigger map
            if (this.triggerMap[char]) {
                // if already searching, terminate the search
                if (this.menuOpened) {
                    this.closeMenu();
                    return;
                }
                // save action trigger
                this.triggerCharPosition = this.elementRef.nativeElement.selectionEnd;
                this.triggerChar = char;
                // open menu
                this.openMenu();
            }
            else {
                // emit search term when menu opened
                if (this.menuOpened) {
                    // update and emit search term
                    this.emitSearchTerm();
                }
            }
        }
        // when delete text backward (using backspace)
        if (this.menuOpened && event.inputType === 'deleteContentBackward') {
            if (this.elementRef.nativeElement.value && this.elementRef.nativeElement.selectionEnd >= this.triggerCharPosition) {
                // update and emit search term
                this.emitSearchTerm();
            }
            else {
                // close menu when no content
                this.closeMenu();
            }
        }
    }
    ngOnInit() {
        this.searchTermDebounceSubscription = this.searchTermDebounce
            .pipe(debounce(() => timer(200)))
            .subscribe(() => {
            this.searchTerm.emit({
                term: this.triggerTerm,
                trigger: this.triggerChar,
            });
        });
    }
    ngOnDestroy() {
        if (this.searchTermDebounceSubscription) {
            this.searchTermDebounceSubscription.unsubscribe();
        }
        // destroy previous overlay
        this.dispose();
    }
    ngOnChanges(changes) {
        // update config trigger map
        if (changes.configs && changes.configs.currentValue) {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            const configs = changes.configs.currentValue;
            this.triggerMap = configs.reduce((mapping, config) => (Object.assign(Object.assign({}, mapping), { [config.trigger]: config })), {});
        }
        // update menu position
        if (changes.cursorPosition && changes.cursorPosition.currentValue && this.menuOpened) {
            this.updateMenuPosition();
        }
    }
    closeMenu() {
        // reset state
        this.menuOpened = false;
        this.triggerCharPosition = 0;
        this.triggerChar = '';
        this.triggerTerm = '';
        // destroy previous overlay
        this.dispose();
    }
    openMenu() {
        // destroy previous overlay
        this.dispose();
        // get config
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const config = this.triggerMap[this.triggerChar];
        if (config) {
            // set opened
            this.menuOpened = true;
            // create overlay
            this.overlayRef = this.overlay.create({
                positionStrategy: this.getPositionStrategy(this.cursorPosition.x + 10, this.cursorPosition.y + 20),
            });
            this.componentRef = this.overlayRef.attach(new ComponentPortal(NgSimpleMarkdownMentionsComponent));
            // set input
            this.componentRef.instance.data = config.data;
            this.componentRef.instance.template = config.template;
            // subscribe to output
            this.countSubscription = this.componentRef.instance.count.subscribe((count) => {
                // set count
                this.itemCount = count;
                // update menu position
                this.updateMenuPosition();
            });
            this.destroySubscription = this.componentRef.instance.destroy.subscribe(() => this.closeMenu());
            this.selectSubscription = this.componentRef.instance.selected.subscribe((item) => {
                // insert content
                this.searchResult.emit({
                    result: config.transform ? config.transform(item) : item,
                    term: this.triggerTerm,
                });
                // close menu
                this.closeMenu();
            });
        }
    }
    getPositionStrategy(x, y) {
        return this.overlay
            .position()
            .flexibleConnectedTo(this.elementRef.nativeElement)
            .withPositions([{
                offsetX: x,
                offsetY: y,
                originX: 'start',
                originY: 'top',
                overlayX: 'start',
                overlayY: 'top',
            }]);
    }
    updateMenuPosition() {
        if (this.overlayRef) {
            // determine flip upwards or downwards
            const rect = this.elementRef.nativeElement.getBoundingClientRect();
            const menuHeight = this.itemCount * 40 + 6; // with item height 40px and padding top 6px
            const menuBottom = rect.y + this.cursorPosition.y + menuHeight;
            if (window.innerHeight < menuBottom) {
                this.overlayRef.updatePositionStrategy(this.getPositionStrategy(this.cursorPosition.x + 10, this.cursorPosition.y - menuHeight));
            }
            else {
                this.overlayRef.updatePositionStrategy(this.getPositionStrategy(this.cursorPosition.x + 10, this.cursorPosition.y + 20));
            }
        }
    }
    emitSearchTerm() {
        if (this.elementRef && this.elementRef.nativeElement) {
            // get current value
            const value = this.elementRef.nativeElement.value || '';
            // get cursor position
            const position = this.elementRef.nativeElement.selectionEnd;
            if (this.triggerCharPosition <= position) {
                this.triggerTerm = value.substring(this.triggerCharPosition, position);
                this.searchTermDebounce.next();
            }
        }
    }
    dispose() {
        if (this.countSubscription) {
            this.countSubscription.unsubscribe();
            this.countSubscription = undefined;
        }
        if (this.destroySubscription) {
            this.destroySubscription.unsubscribe();
            this.destroySubscription = undefined;
        }
        if (this.selectSubscription) {
            this.selectSubscription.unsubscribe();
            this.selectSubscription = undefined;
        }
        if (this.componentRef) {
            this.componentRef.destroy();
            this.componentRef = undefined;
        }
        if (this.overlayRef) {
            this.overlayRef.dispose();
            this.overlayRef = undefined;
        }
    }
}
NgSimpleMarkdownMentionDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngSimpleMarkdownMention]',
            },] }
];
NgSimpleMarkdownMentionDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Overlay }
];
NgSimpleMarkdownMentionDirective.propDecorators = {
    configs: [{ type: Input, args: ['ngSimpleMarkdownMention',] }],
    cursorPosition: [{ type: Input }],
    searchResult: [{ type: Output }],
    searchTerm: [{ type: Output }],
    onInput: [{ type: HostListener, args: ['input', ['$event'],] }]
};

class NgSimpleMarkdownEditorComponent {
    constructor(overlay) {
        this.overlay = overlay;
        this.height = '400px';
        this.historySteps = 10;
        this.placeholder = '';
        this.resize = true;
        // for file upload
        this.files = new EventEmitter(true);
        this.mentionSearchTerm = new EventEmitter(true);
        // editor state
        this.isDisabled = false;
        this.isFullscreen = false;
        this.isWriteMode = true;
        this.screenSizeBreakpoint = 600;
        // editor model
        this.markdown = new FormControl('');
        this.shadowContentUpper = '';
        this.shadowContentLower = '';
        // resize
        this.resizeOffset = 0;
        this.resizeStart = 0;
        this.resizeScrollStart = 0;
        // redo/undo state
        this.isPerformingUndoRedo = false;
        this.undoStack = [];
        this.redoStack = [];
    }
    keyboardEvent(event) {
        // get key press
        const key = (event.key || '').toLowerCase();
        // check action
        switch (key) {
            case 'escape': {
                if (this.isFullscreen && !this.isOpenedMentionMenu) {
                    this.fullscreen();
                }
                break;
            }
            case 'z': {
                if (this.historySteps > 0 && (event.ctrlKey || event.metaKey)) {
                    if (event.shiftKey) {
                        this.redo();
                    }
                    else {
                        this.undo();
                    }
                    // prevent the default undo/redo handling
                    event.preventDefault();
                    event.stopPropagation();
                }
                break;
            }
            case 'arrowdown': /* cspell: disable-line */
            case 'arrowleft': /* cspell: disable-line */
            case 'arrowright': /* cspell: disable-line */
            case 'arrowup': { /* cspell: disable-line */
                // update cursor
                this.updateCursor();
                break;
            }
        }
    }
    ngOnInit() {
        // set up resizing functions to global scope
        window['editorResizing'] = (e) => {
            this.onResizing(e);
        };
        window['editorResizeEnd'] = () => {
            this.onResizeEnd();
        };
        // watch for control value change
        this.controlSubscription = this.markdown.valueChanges
            .pipe(debounce(() => timer(200)))
            .subscribe((value) => {
            // save history
            if (this.historySteps > 0) {
                if (!this.isPerformingUndoRedo) {
                    this.saveUndo();
                }
                // reset state
                this.isPerformingUndoRedo = false;
            }
            // trigger changes
            if (this.fnOnChange) {
                this.fnOnChange(value);
            }
            if (this.fnOnTouched) {
                this.fnOnTouched();
            }
        });
    }
    ngOnDestroy() {
        if (this.controlSubscription) {
            this.controlSubscription.unsubscribe();
        }
        if (this.fullScreenControlSubscription) {
            this.fullScreenControlSubscription.unsubscribe();
        }
    }
    get isMobile() {
        return window.innerWidth < this.screenSizeBreakpoint;
    }
    get editorHeight() {
        const toolbarHeight = this.isMobile ? 81 : 41;
        const borderSizeAdjust = 2;
        return `${parseInt(this.height, 10) + this.resizeOffset - toolbarHeight - borderSizeAdjust}px`;
    }
    get isInFullscreen() {
        return !!this.fullScreenComponentRef && !!this.fullScreenComponentRef.instance;
    }
    get isOpenedMentionMenu() {
        return this.mentionDirective.menuOpened;
    }
    writeValue(markdown) {
        if (markdown !== null && markdown !== undefined) {
            this.markdown.setValue(markdown);
        }
    }
    registerOnChange(fn) {
        this.fnOnChange = fn;
    }
    registerOnTouched(fn) {
        this.fnOnTouched = fn;
    }
    setDisabledState(isDisabled) {
        if (isDisabled) {
            this.markdown.disable();
        }
        this.isDisabled = isDisabled;
    }
    onDrop(event) {
        if (event.dataTransfer.files.length > 0) {
            this.files.emit(Array.from(event.dataTransfer.files));
            event.stopPropagation();
            event.preventDefault();
        }
    }
    onPaste(event) {
        if (event.clipboardData.files.length > 0) {
            event.stopPropagation();
            event.preventDefault();
            this.files.emit(Array.from(event.clipboardData.files));
        }
        // update cursor
        this.updateCursor();
    }
    onInput() {
        // update cursor
        this.updateCursor();
    }
    onResizeStart(event) {
        if (this.resize && this.isWriteMode) {
            if (this.resizeStart === 0) {
                this.resizeStart = event.clientY;
            }
            if (this.resizeScrollStart === 0) {
                const rect = this.editingContainerElement.nativeElement.getBoundingClientRect();
                this.resizeScrollStart = rect.y;
            }
            document.documentElement.addEventListener('mousemove', window['editorResizing'], false);
            document.documentElement.addEventListener('mouseup', window['editorResizeEnd'], false);
        }
    }
    onResizing(event) {
        const rect = this.editingContainerElement.nativeElement.getBoundingClientRect();
        this.resizeOffset = event.clientY + (this.resizeScrollStart - rect.y) - this.resizeStart;
        if (this.resizeOffset < 0) {
            this.resizeOffset = 0;
        }
    }
    onResizeEnd() {
        document.documentElement.removeEventListener('mousemove', window['editorResizing'], false);
        document.documentElement.removeEventListener('mouseup', window['editorResizeEnd'], false);
    }
    onEditorScroll(event) {
        this.shadowEditorElement.nativeElement.scrollTop = event.target.scrollTop;
    }
    updateCursor() {
        setTimeout(() => {
            // get selection range
            const position = this.textareaElement.nativeElement.selectionEnd;
            // markdown value
            const value = this.markdown.value;
            // set content
            this.shadowContentUpper = value.substring(0, position);
            this.shadowContentLower = value.substring(position);
            // update cursor position
            this.cursorPosition = {
                x: this.shadowCursorElement.nativeElement.offsetLeft,
                y: this.shadowCursorElement.nativeElement.offsetTop,
            };
        });
    }
    fullscreen() {
        // close all mention menu
        this.mentionDirective.closeMenu();
        if (this.isFullscreen) {
            // exit full screen
            if (this.fullScreenControlSubscription) {
                this.fullScreenControlSubscription.unsubscribe();
            }
            if (this.fullScreenComponentRef) {
                this.fullScreenComponentRef.destroy();
            }
            if (this.fullScreenOverlayRef) {
                this.fullScreenOverlayRef.dispose();
            }
        }
        else {
            // create overlay
            this.fullScreenOverlayRef = this.overlay.create();
            this.fullScreenComponentRef = this.fullScreenOverlayRef.attach(new ComponentPortal(NgSimpleMarkdownEditorComponent));
            this.fullScreenComponentRef.onDestroy(() => {
                // set editor selection
                this.textareaElement.nativeElement.setSelectionRange(this.fullScreenComponentRef.instance.textareaElement.nativeElement.selectionStart, this.fullScreenComponentRef.instance.textareaElement.nativeElement.selectionEnd);
                // update cursor
                this.updateCursor();
                // unset component reference
                this.fullScreenComponentRef = undefined;
            });
            // set input
            this.fullScreenComponentRef.instance.height = this.height;
            this.fullScreenComponentRef.instance.historySteps = this.historySteps;
            this.fullScreenComponentRef.instance.placeholder = this.placeholder;
            this.fullScreenComponentRef.instance.resize = this.resize;
            // for file upload
            this.fullScreenComponentRef.instance.files = this.files;
            // for mentions
            this.fullScreenComponentRef.instance.mentionConfigs = this.mentionConfigs;
            this.fullScreenComponentRef.instance.mentionSearchTerm = this.mentionSearchTerm;
            // editor state
            this.fullScreenComponentRef.instance.isDisabled = this.isDisabled;
            this.fullScreenComponentRef.instance.isFullscreen = true;
            this.fullScreenComponentRef.instance.isWriteMode = this.isWriteMode;
            this.fullScreenComponentRef.instance.cursorPosition = this.cursorPosition;
            this.fullScreenComponentRef.instance.screenSizeBreakpoint = this.screenSizeBreakpoint;
            // editor model
            this.fullScreenComponentRef.instance.markdown.setValue(this.markdown.value);
            this.fullScreenComponentRef.instance.shadowContentUpper = this.shadowContentUpper;
            this.fullScreenComponentRef.instance.shadowContentLower = this.shadowContentLower;
            // redo/undo state
            this.fullScreenComponentRef.instance.isPerformingUndoRedo = this.isPerformingUndoRedo;
            this.fullScreenComponentRef.instance.undoStack = this.undoStack;
            this.fullScreenComponentRef.instance.redoStack = this.redoStack;
            // full screen
            this.fullScreenComponentRef.instance.fullScreenOverlayRef = this.fullScreenOverlayRef;
            // this.fullScreenComponentRef.instance.fullScreenComponentRef = this.fullScreenComponentRef; <<-- don't un-comment this
            setTimeout(() => {
                // set editor selection
                this.fullScreenComponentRef.instance.textareaElement.nativeElement.setSelectionRange(this.textareaElement.nativeElement.selectionStart, this.textareaElement.nativeElement.selectionEnd);
                // update cursor
                this.fullScreenComponentRef.instance.updateCursor();
            });
            // pipe overlay markdown
            this.fullScreenControlSubscription = this.fullScreenComponentRef.instance.markdown.valueChanges
                .pipe(debounce(() => timer(50)))
                .subscribe((value) => {
                // set value
                this.markdown.setValue(value);
                // trigger changes
                if (this.fnOnChange) {
                    this.fnOnChange(value);
                }
                if (this.fnOnTouched) {
                    this.fnOnTouched();
                }
            });
        }
    }
    onSearchResult(result) {
        this.insertContent(result.result, result.term.length);
    }
    onSearchTerm(term) {
        this.mentionSearchTerm.emit(term);
    }
    // formatting function
    insertLink() {
        // get url
        const description = prompt('Enter url text');
        const url = prompt('Enter url');
        if (url) {
            // construct string
            this.insertContent(`[${description}](${url})`);
            // save history
            this.saveUndo();
            // update cursor
            this.updateCursor();
        }
    }
    insertPhoto() {
        // get image url
        const description = prompt('Enter image description (optional)');
        const url = prompt('Enter image url');
        if (url) {
            // construct string
            this.insertContent(`![${description}](${url})`);
            // save history
            this.saveUndo();
            // update cursor
            this.updateCursor();
        }
    }
    bold() {
        // get selection range
        const start = this.textareaElement.nativeElement.selectionStart;
        const end = this.textareaElement.nativeElement.selectionEnd;
        // markdown value
        const value = this.markdown.value;
        // construct string
        const result = ''.concat(value.substring(0, start), `**${value.substring(start, end)}**`, value.substr(end));
        // update editor
        this.markdown.setValue(result);
        // save history
        this.saveUndo();
        // update cursor
        this.updateCursor();
    }
    italic() {
        // get selection range
        const start = this.textareaElement.nativeElement.selectionStart;
        const end = this.textareaElement.nativeElement.selectionEnd;
        // markdown value
        const value = this.markdown.value;
        // construct string
        const result = ''.concat(value.substring(0, start), `*${value.substring(start, end)}*`, value.substr(end));
        // update editor
        this.markdown.setValue(result);
        // save history
        this.saveUndo();
        // update cursor
        this.updateCursor();
    }
    quote() {
        // get selection range
        const start = this.textareaElement.nativeElement.selectionStart;
        const end = this.textareaElement.nativeElement.selectionEnd;
        // markdown value
        const value = this.markdown.value;
        // manipulate strings
        const lines = value.substring(start, end).split('\n');
        const modifiedString = start === end ? ['> '] : lines.reduce((resultLines, line) => [
            ...resultLines,
            `> ${line}`,
        ], []);
        // construct string
        const result = ''.concat(value.substring(0, start), modifiedString.join('\n'), value.substr(end));
        // update editor
        this.markdown.setValue(result);
        // save history
        this.saveUndo();
        // update cursor
        this.updateCursor();
    }
    code() {
        // get selection range
        const start = this.textareaElement.nativeElement.selectionStart;
        const end = this.textareaElement.nativeElement.selectionEnd;
        // markdown value
        const value = this.markdown.value;
        // construct string
        const result = ''.concat(value.substring(0, start), `\`\`\`\n${value.substring(start, end)}\n\`\`\``, value.substr(end));
        // update editor
        this.markdown.setValue(result);
        // save history
        this.saveUndo();
        // update cursor
        this.updateCursor();
    }
    bulletedList() {
        // get selection range
        const start = this.textareaElement.nativeElement.selectionStart;
        const end = this.textareaElement.nativeElement.selectionEnd;
        // markdown value
        const value = this.markdown.value;
        // manipulate strings
        const lines = value.substring(start, end).split('\n');
        const modifiedString = start === end ? ['* '] : lines.reduce((resultLines, line) => [
            ...resultLines,
            `* ${line}`,
        ], []);
        // construct string
        const result = ''.concat(value.substring(0, start), modifiedString.join('\n'), value.substr(end));
        // update editor
        this.markdown.setValue(result);
        // save history
        this.saveUndo();
        // update cursor
        this.updateCursor();
    }
    numberedList() {
        // get selection range
        const start = this.textareaElement.nativeElement.selectionStart;
        const end = this.textareaElement.nativeElement.selectionEnd;
        // markdown value
        const value = this.markdown.value;
        // manipulate strings
        const lines = value.substring(start, end).split('\n');
        const modifiedString = start === end ? ['1. '] : lines.reduce((resultLines, line, index) => [
            ...resultLines,
            `${index + 1}. ${line}`,
        ], []);
        // construct string
        const result = ''.concat(value.substring(0, start), modifiedString.join('\n'), value.substr(end));
        // update editor
        this.markdown.setValue(result);
        // save history
        this.saveUndo();
        // update cursor
        this.updateCursor();
    }
    taskList() {
        // get selection range
        const start = this.textareaElement.nativeElement.selectionStart;
        const end = this.textareaElement.nativeElement.selectionEnd;
        // markdown value
        const value = this.markdown.value;
        // manipulate strings
        const lines = value.substring(start, end).split('\n');
        const modifiedString = start === end ? ['- [ ] '] : lines.reduce((resultLines, line) => [
            ...resultLines,
            `- [ ] ${line}`,
        ], []);
        // construct string
        const result = ''.concat(value.substring(0, start), modifiedString.join('\n'), value.substr(end));
        // update editor
        this.markdown.setValue(result);
        // save history
        this.saveUndo();
        // update cursor
        this.updateCursor();
    }
    insertContent(content, skipBackwardLength = 0) {
        // insert to fullscreen editor
        if (this.isInFullscreen) {
            return this.fullScreenComponentRef.instance.insertContent(content, skipBackwardLength);
        }
        // insert to this editor
        if (this.textareaElement) {
            // get selection range
            const start = this.textareaElement.nativeElement.selectionStart;
            const end = this.textareaElement.nativeElement.selectionEnd;
            // markdown value
            const value = this.markdown.value;
            // construct string
            const result = ''.concat(value.substring(0, start - skipBackwardLength), content, value.substr(end));
            // update editor
            this.markdown.setValue(result);
            // save history
            this.saveUndo();
            // get new cursor position
            const position = start - skipBackwardLength + content.length;
            // update selection range
            this.textareaElement.nativeElement.setSelectionRange(position, position);
            // update cursor
            this.updateCursor();
        }
        return content.length;
    }
    // editor history
    saveUndo() {
        if (this.textareaElement && this.historySteps > 0) {
            // save editor state
            this.undoStack.unshift(new EditorHistory(this.markdown.value, this.textareaElement.nativeElement.selectionStart, this.textareaElement.nativeElement.selectionEnd));
            if (this.undoStack.length > this.historySteps) {
                for (let i = 0; i < this.undoStack.length - this.historySteps; i++) {
                    this.undoStack.pop();
                }
            }
            // clear redo stack
            this.redoStack = [];
        }
    }
    undo() {
        if (this.textareaElement && this.historySteps > 0 && this.undoStack.length > 0) {
            // move the first element from undo to redo
            this.redoStack.unshift(this.undoStack.shift());
            // restore editor state
            const [current] = this.undoStack;
            if (current) {
                this.isPerformingUndoRedo = true;
                this.markdown.setValue(current.content || '');
                this.textareaElement.nativeElement.setSelectionRange(current.selectionStart, current.selectionEnd);
                this.updateCursor();
            }
        }
    }
    redo() {
        if (this.textareaElement && this.historySteps > 0 && this.redoStack.length > 0) {
            // move the first element from redo to undo
            this.undoStack.unshift(this.redoStack.shift());
            // restore editor state
            const [current] = this.undoStack;
            if (current) {
                this.isPerformingUndoRedo = true;
                this.markdown.setValue(current.content || '');
                this.textareaElement.nativeElement.setSelectionRange(current.selectionStart, current.selectionEnd);
                this.updateCursor();
            }
        }
    }
}
NgSimpleMarkdownEditorComponent.decorators = [
    { type: Component, args: [{
                providers: [
                    {
                        multi: true,
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => NgSimpleMarkdownEditorComponent),
                    },
                ],
                selector: 'ng-simple-markdown-editor',
                template: "<div class=\"ng-simple-markdown-editor\" [class.ng-simple-markdown-editor--fullscreen]=\"isFullscreen\">\n\n\t<div class=\"ng-simple-markdown-editor__toolbar\">\n\n\t\t<div class=\"ng-simple-markdown-editor__view-mode\">\n\t\t\t<button mat-button [disabled]=\"isDisabled\" [color]=\"isWriteMode ? 'primary' : ''\" (click)=\"isWriteMode = true\">Write</button>\n\t\t\t<button mat-button [disabled]=\"isDisabled\" [color]=\"isWriteMode ? '' : 'primary'\" (click)=\"isWriteMode = false\">Preview</button>\n\t\t</div>\n\n\t\t<div class=\"ng-simple-markdown-editor__spacer\"></div>\n\n\t\t<div class=\"ng-simple-markdown-editor__format\" *ngIf=\"isWriteMode\">\n\t\t\t<button mat-icon-button [disabled]=\"isDisabled\" (click)=\"insertLink()\"><mat-icon>link</mat-icon></button>\n\t\t\t<button mat-icon-button [disabled]=\"isDisabled\" (click)=\"insertPhoto()\"><mat-icon>insert_photo</mat-icon></button>\n\t\t\t<button mat-icon-button [disabled]=\"isDisabled\" (click)=\"bold()\"><mat-icon>format_bold</mat-icon></button>\n\t\t\t<button mat-icon-button [disabled]=\"isDisabled\" (click)=\"italic()\"><mat-icon>format_italic</mat-icon></button>\n\t\t\t<button mat-icon-button [disabled]=\"isDisabled\" (click)=\"quote()\"><mat-icon>format_quote</mat-icon></button>\n\t\t\t<button mat-icon-button [disabled]=\"isDisabled\" (click)=\"code()\"><mat-icon>code</mat-icon></button>\n\t\t\t<button mat-icon-button [disabled]=\"isDisabled\" (click)=\"bulletedList()\"><mat-icon>format_list_bulleted</mat-icon></button>\n\t\t\t<button mat-icon-button [disabled]=\"isDisabled\" (click)=\"numberedList()\"><mat-icon>format_list_numbered</mat-icon></button>\n\t\t\t<button mat-icon-button [disabled]=\"isDisabled\" (click)=\"taskList()\"><mat-icon>assignment_turned_in</mat-icon></button>\n\t\t\t<button mat-icon-button [disabled]=\"isDisabled\" (click)=\"fullscreen()\"><mat-icon>fullscreen</mat-icon></button>\n\t\t</div>\n\n\t</div>\n\n\t<div #editingContainerElement class=\"ng-simple-markdown-editor__editing\" [style.height]=\"editorHeight\">\n\n\t\t<ng-container *ngIf=\"isWriteMode; then editorView; else previewView\"></ng-container>\n\n\t\t<ng-template #editorView>\n\t\t\t<textarea #textareaElement\n\t\t\t\t(click)=\"updateCursor()\"\n\t\t\t\t(drop)=\"onDrop($event)\"\n\t\t\t\t(input)=\"onInput()\"\n\t\t\t\t(paste)=\"onPaste($event)\"\n\t\t\t\t(scroll)=\"onEditorScroll($event)\"\n\t\t\t\t(searchResult)=\"onSearchResult($event)\"\n\t\t\t\t(searchTerm)=\"onSearchTerm($event)\"\n\t\t\t\t[cursorPosition]=\"cursorPosition\"\n\t\t\t\t[formControl]=\"markdown\"\n\t\t\t\t[ngSimpleMarkdownMention]=\"mentionConfigs\"\n\t\t\t\t[placeholder]=\"placeholder\"\n\t\t\t></textarea>\n\t\t\t<pre #shadowEditorElement>{{shadowContentUpper}}<span class=\"cursor\" #shadowCursorElement></span>{{shadowContentLower}}</pre>\n\t\t</ng-template>\n\n\t\t<ng-template #previewView>\n\t\t\t<iframe ngSimpleMarkdownViewer [markdown]=\"markdown.value\"></iframe>\n\t\t</ng-template>\n\n\t</div>\n\n</div>\n<div *ngIf=\"resize && isWriteMode\" class=\"resize-handle\" (mousedown)=\"onResizeStart($event)\"></div>\n",
                styles: [".ng-simple-markdown-editor{align-items:stretch;background-color:#fff;border:1px solid #ccc;display:flex;flex-direction:column;height:100%;justify-content:flex-start}.ng-simple-markdown-editor--fullscreen{bottom:0!important;left:0!important;margin:0!important;position:fixed!important;right:0!important;top:0!important;z-index:9999!important}.ng-simple-markdown-editor__toolbar{align-items:stretch;border-bottom:1px solid #ccc;display:flex;justify-content:flex-start;padding:0 2px}@media (min-width:600px){.ng-simple-markdown-editor__toolbar{flex-direction:row;height:40px}}@media (max-width:600px){.ng-simple-markdown-editor__toolbar{flex-direction:column;height:80px}}.ng-simple-markdown-editor__spacer{flex:1 1 auto}@media (max-width:600px){.ng-simple-markdown-editor__spacer{display:none}}.ng-simple-markdown-editor__view-mode{display:flex;flex-direction:row;height:40px}@media (min-width:600px){.ng-simple-markdown-editor__view-mode{align-items:center;justify-content:flex-start}}@media (max-width:600px){.ng-simple-markdown-editor__view-mode{align-items:center;justify-content:space-evenly}}.ng-simple-markdown-editor__format{display:flex;flex-direction:row;height:40px;justify-content:flex-start}@media (min-width:600px){.ng-simple-markdown-editor__format{align-items:center}}@media (max-width:600px){.ng-simple-markdown-editor__format{-webkit-overflow-scrolling:touch;align-items:center;overflow-x:auto}}.ng-simple-markdown-editor__editing{align-items:flex-start;display:flex;flex:1 1 auto;flex-direction:row;justify-content:flex-start;overflow:auto;position:relative}.ng-simple-markdown-editor__editing iframe{-o-object-fit:contain;-o-object-position:center center;border:none;object-fit:contain;object-position:center center;padding:10px;width:calc(100% - 20px)!important}.ng-simple-markdown-editor__editing pre,.ng-simple-markdown-editor__editing textarea{border:none;bottom:0;font-family:Lucida Console,Monaco,monospace;font-size:14px;height:calc(100% - 20px)!important;left:0;line-height:1.4;margin:0;overflow-y:auto;padding:10px;position:absolute;resize:none;right:0;top:0;white-space:-moz-pre-wrap;white-space:-o-pre-wrap;white-space:pre-wrap;width:calc(100% - 20px)!important;word-wrap:break-word}.ng-simple-markdown-editor__editing pre:active,.ng-simple-markdown-editor__editing pre:focus,.ng-simple-markdown-editor__editing textarea:active,.ng-simple-markdown-editor__editing textarea:focus{-moz-appearance:none;-webkit-appearance:none;appearance:none;outline:0}.ng-simple-markdown-editor__editing pre{color:transparent;pointer-events:none}.ng-simple-markdown-editor__editing pre .cursor{border:1px solid transparent}.resize-handle{background-color:#ccc;cursor:row-resize;height:2px;width:100%}"]
            },] }
];
NgSimpleMarkdownEditorComponent.ctorParameters = () => [
    { type: Overlay }
];
NgSimpleMarkdownEditorComponent.propDecorators = {
    editingContainerElement: [{ type: ViewChild, args: ['editingContainerElement',] }],
    shadowCursorElement: [{ type: ViewChild, args: ['shadowCursorElement',] }],
    shadowEditorElement: [{ type: ViewChild, args: ['shadowEditorElement',] }],
    textareaElement: [{ type: ViewChild, args: ['textareaElement',] }],
    mentionDirective: [{ type: ViewChild, args: [NgSimpleMarkdownMentionDirective,] }],
    height: [{ type: Input }],
    historySteps: [{ type: Input }],
    placeholder: [{ type: Input }],
    resize: [{ type: Input }],
    files: [{ type: Output }],
    mentionConfigs: [{ type: Input }],
    mentionSearchTerm: [{ type: Output }],
    keyboardEvent: [{ type: HostListener, args: ['document:keydown', ['$event'],] }]
};

const MarkdownItHashTag = (md) => {
    const parserId = 'hash-tag';
    const parser = (state, silent) => {
        // get parser input
        const matches = new RegExp(/#\w+/).exec(state.src.slice(state.pos));
        if (
        // if nothing match
        !matches ||
            // check if matched string is start of input
            (matches.index > 0 &&
                matches.input[matches.index - 1]) ||
            // if not separated by space or start of line
            (state.pos > 0 &&
                state.src[state.pos - 1] !== ' ' &&
                state.src[state.pos - 1] !== '\n')) {
            return false;
        }
        // advance the cursor
        state.pos += matches[0].length;
        // no token in silent mode
        if (silent) {
            return true;
        }
        // generate token
        const token = state.push(parserId, '', 0);
        token.meta = { matches };
        return true;
    };
    const render = (tokens, id) => {
        // get matches
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
        const matches = tokens[id].meta.matches;
        const tag = matches[0];
        // render result
        return `<a href="#" data-mention-tag="${tag}">${tag}</a>`;
    };
    // register parser & render
    md.inline.ruler.push(parserId, parser);
    md.renderer.rules[parserId] = render;
};

const MarkdownItMentionWithId = (md) => {
    const parserId = 'mention-with-id';
    const parser = (state, silent) => {
        // get parser input
        const matches = new RegExp(/@\(([0-9a-f-]+)\|([\w.\-_]+)\)/i).exec(state.src.slice(state.pos));
        if (
        // if nothing match
        !matches ||
            // check if matched string is start of input
            (matches.index > 0 &&
                matches.input[matches.index - 1]) ||
            // if not separated by space or start of line
            (state.pos > 0 &&
                state.src[state.pos - 1] !== ' ' &&
                state.src[state.pos - 1] !== '\n')) {
            return false;
        }
        // advance the cursor
        state.pos += matches[0].length;
        // no token in silent mode
        if (silent) {
            return true;
        }
        // generate token
        const token = state.push(parserId, '', 0);
        token.meta = { matches };
        return true;
    };
    const render = (tokens, id) => {
        // get matches
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
        const matches = tokens[id].meta.matches;
        // get matched mention
        const userId = matches[1];
        const name = matches[2];
        // render result
        return `<a href="#" data-mention-id="${userId}" data-mention-name="${name}">${`@${name}`}</a>`;
    };
    // register parser & render
    md.inline.ruler.push(parserId, parser);
    md.renderer.rules[parserId] = render;
};

const MarkdownItMentions = (md) => {
    const parserId = 'mention';
    const parser = (state, silent) => {
        // get parser input
        const matches = new RegExp(/@([\w.\-_]+)/i).exec(state.src.slice(state.pos));
        if (
        // if nothing match
        !matches ||
            // check if matched string is start of input
            (matches.index > 0 &&
                matches.input[matches.index - 1]) ||
            // if not separated by space or start of line
            (state.pos > 0 &&
                state.src[state.pos - 1] !== ' ' &&
                state.src[state.pos - 1] !== '\n')) {
            return false;
        }
        // advance the cursor
        state.pos += matches[0].length;
        // no token in silent mode
        if (silent) {
            return true;
        }
        // generate token
        const token = state.push(parserId, '', 0);
        token.meta = { matches };
        return true;
    };
    const render = (tokens, id) => {
        // get matches
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
        const matches = tokens[id].meta.matches;
        // get matched mention
        const name = matches[1];
        // render result
        return `<a href="#" data-mention-name="${name}">${`@${name}`}</a>`;
    };
    // register parser & render
    md.inline.ruler.push(parserId, parser);
    md.renderer.rules[parserId] = render;
};

const MarkdownItTitle = (md) => {
    const parserId = 'title';
    const parser = (state, silent) => {
        // get parser input
        const matches = new RegExp(/\{(.+)\|\|(.+)\}/).exec(state.src.slice(state.pos));
        if (
        // if nothing match
        !matches ||
            // check if matched string is start of input
            (matches.index > 0 &&
                matches.input[matches.index - 1])) {
            return false;
        }
        // advance the cursor
        state.pos += matches[0].length;
        // no token in silent mode
        if (silent) {
            return true;
        }
        // generate token
        const token = state.push(parserId, '', 0);
        token.meta = { matches };
        return true;
    };
    const render = (tokens, id) => {
        // get matches
        // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
        const matches = tokens[id].meta.matches;
        // get matched mention
        const text = matches[1];
        const title = matches[2];
        // render result
        return `<span title="${title}">${text}</a>`;
    };
    // register parser & render
    md.inline.ruler.push(parserId, parser);
    md.renderer.rules[parserId] = render;
};

class NgSimpleMarkdownService {
    constructor() {
        // init instance
        this.markdownIt = new MarkdownIt({
            breaks: true,
            linkify: true,
        });
        // use plugins
        this.markdownIt.use(MarkdownItHashTag);
        this.markdownIt.use(MarkdownItMentions);
        this.markdownIt.use(MarkdownItMentionWithId);
        this.markdownIt.use(MarkdownItTitle);
        this.markdownIt.use(TaskList, { enabled: true });
    }
    parse(markdown) {
        return this.markdownIt.render(markdown);
    }
}
NgSimpleMarkdownService.decorators = [
    { type: Injectable }
];
NgSimpleMarkdownService.ctorParameters = () => [];

class NgSimpleMarkdownTextPreviewPipe {
    constructor(ngSimpleMarkdownService) {
        this.ngSimpleMarkdownService = ngSimpleMarkdownService;
    }
    transform(markdown) {
        if (markdown) {
            const html = this.ngSimpleMarkdownService.parse(markdown);
            const placeholder = document.createElement('div');
            placeholder.innerHTML = this.filterHTML(html);
            return placeholder.textContent;
        }
        return '';
    }
    filterHTML(html) {
        const dom = (new DOMParser()).parseFromString(html, 'text/html');
        // remove images
        const imgTags = Array.from(dom.querySelectorAll('img'));
        for (const imgTag of imgTags) {
            imgTag.remove();
        }
        return (new XMLSerializer()).serializeToString(dom);
    }
}
NgSimpleMarkdownTextPreviewPipe.decorators = [
    { type: Pipe, args: [{
                name: 'ngSimpleMarkdownTextPreview',
            },] }
];
NgSimpleMarkdownTextPreviewPipe.ctorParameters = () => [
    { type: NgSimpleMarkdownService }
];

class NgSimpleMarkdownViewerDirective {
    constructor(elementRef, ngSimpleMarkdownService, sanitizer) {
        this.elementRef = elementRef;
        this.ngSimpleMarkdownService = ngSimpleMarkdownService;
        this.sanitizer = sanitizer;
        this.mentions = new EventEmitter(true);
        this.mentionWithIds = new EventEmitter(true);
        this.ready = new EventEmitter(true);
        this.tags = new EventEmitter(true);
        this.taskLists = new EventEmitter(true);
        this.border = 'none';
        this.iframeHeight = '0px';
        this.iframeWidth = '0px';
        this.iframeStyle = `
		<style type="text/css">
		body {
			font-family: Roboto, "Helvetica Neue", sans-serif; /* cspell: disable-line */
			margin: 0;
			word-wrap: break-word;
		}
		a {
			border-radius: 0;
			color: #337ab7;
			font-weight: normal;
			text-decoration: none;
			width: 100%;
			word-wrap: break-all;
		}
		a:focus, a:hover {
			background-color: transparent;
			color: #23527c;
			text-decoration: underline;
		}
		blockquote {
			border-left: 5px #cccccc solid;
			margin: 13px 40px;
			padding: 2px 8px 2px 20px;
		}
		code {
			background-color: #fcedea;
			border-radius: 3px;
			color: #c0341d;
			font-size: 90%;
			padding: 2px 4px;
		}
		pre {
			border-radius: 2px;
			border: 1px solid #e5e5e5;
			display: block;
			line-height: 1.6em;
			margin: 16px 4px;
			overflow-x: auto;
			padding: 10px;
			word-break: break-all;
			word-wrap: break-word;
		}
		pre code {
			background-color: transparent;
			color: #333;
		}
		table {
			border-collapse: collapse;
			border-spacing: 0;
			border: 1px solid #ddd;
			margin: 5px;
		}
		table tr:nth-child(even) {
			background-color: #efefef;
		}
		table th {
			border: 1px solid #ddd;
			font-weight: bold;
			padding: 10px;
		}
		table td {
			border: 1px solid #ddd;
			padding: 10px;
		}
		h1, h2, h3, h4, h5 {
			line-height: 1;
		}
		img {
			border: 1px #e0e0e0 solid;
			display: block;
			height: auto;
			max-width: calc(100% - 10px);
			padding: 4px;
		}
		p {
			margin: 0;
		}
		p + p {
			padding-top: 16px;
		}
		.contains-task-list {
			list-style: none;
			padding-left: 20px;
		}
		</style>
	`;
    }
    getContentDocument() {
        if (!this.iframeDocument) {
            // get iframe html element
            const iframeElement = this.elementRef.nativeElement;
            // get content document
            this.iframeDocument = iframeElement.contentWindow.document || iframeElement.contentDocument;
        }
        return this.iframeDocument;
    }
    onLoad() {
        return __awaiter(this, void 0, void 0, function* () {
            // update height
            yield this.updateSize();
            // search for mentions
            const mentions = this.getContentDocument().querySelectorAll('[data-mention-name]');
            if (mentions.length > 0) {
                mentions.forEach((mention) => {
                    mention.addEventListener('click', (event) => {
                        const id = mention.dataset.mentionId;
                        const name = mention.dataset.mentionName;
                        if (id) {
                            this.mentionWithIds.emit([id, name]);
                        }
                        else {
                            this.mentions.emit(name);
                        }
                        event.preventDefault();
                        event.stopPropagation();
                    });
                });
            }
            // search for hash tags
            const hashTags = this.getContentDocument().querySelectorAll('[data-mention-tag]');
            if (hashTags.length > 0) {
                hashTags.forEach((hashTag) => {
                    hashTag.addEventListener('click', (event) => {
                        this.tags.emit(hashTag.dataset.mentionTag);
                        event.preventDefault();
                        event.stopPropagation();
                    });
                });
            }
            // search for task lists
            const taskListItems = this.getContentDocument().querySelectorAll('.task-list-item-checkbox');
            if (taskListItems.length > 0) {
                taskListItems.forEach((taskListItem, index) => {
                    taskListItem.addEventListener('click', (event) => {
                        if (this.markdown) {
                            const normalized = ['* [ ] ', '* [x] ', '- [x] '].reduce((str, deli) => str.split(deli).join('- [ ] '), this.markdown);
                            const position = new Array(index).fill(0).reduce((pos) => normalized.indexOf('- [ ] ', pos + 1), normalized.indexOf('- [ ] '));
                            const result = this.markdown.substring(0, position + 3)
                                + (this.markdown.charAt(position + 3) === ' ' ? 'x' : ' ')
                                + this.markdown.substring(position + 4);
                            this.taskLists.emit(result);
                        }
                        event.preventDefault();
                        event.stopPropagation();
                    });
                });
            }
            // notify done
            this.ready.emit(true);
        });
    }
    ngOnChanges(changes) {
        // when new email bind then re-paint iframe content
        if (changes.markdown) {
            // notify loading
            this.ready.emit(false);
            // additional styles
            const additionalStyles = this.styles ? `<style type="text/css">${this.styles}</style>` : '';
            // parse markdown
            const html = this.iframeStyle + additionalStyles + this.ngSimpleMarkdownService.parse(changes.markdown.currentValue);
            // bypass html security check
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            const secureHTML = this.sanitizer.bypassSecurityTrustHtml(html);
            // filter html content
            // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
            const filteredHTML = this.filterHTML(secureHTML.changingThisBreaksApplicationSecurity);
            // write html
            this.getContentDocument().open();
            this.getContentDocument().write(filteredHTML);
            this.getContentDocument().close();
        }
    }
    updateSize() {
        return new Promise((resolve) => {
            // set height
            setTimeout(() => {
                // get height
                const htmlHeight = this.getContentDocument().querySelector('html').scrollHeight;
                const bodyHeight = this.getContentDocument().querySelector('body').scrollHeight;
                // get width
                const htmlWidth = this.getContentDocument().querySelector('html').scrollWidth;
                const bodyWidth = this.getContentDocument().querySelector('body').scrollWidth;
                // update iframe size
                this.iframeHeight = `${Math.max(htmlHeight, bodyHeight)}px`;
                this.iframeWidth = `${Math.max(htmlWidth, bodyWidth)}px`;
                resolve();
            });
        });
    }
    filterHTML(html) {
        // parse HTML
        const dom = (new DOMParser()).parseFromString(html, 'text/html');
        // hide excess content
        const htmlTag = dom.querySelector('html');
        if (htmlTag) {
            htmlTag.style.overflow = 'hidden';
        }
        // set link target
        const links = dom.querySelectorAll('a');
        for (let i = 0; i < links.length; i++) {
            links.item(i).setAttribute('target', '_blank');
        }
        // set mention tag url
        if (this.getTagUrl) {
            const mentionTags = dom.querySelectorAll('[data-mention-tag]');
            for (let i = 0; i < mentionTags.length; i++) {
                const value = mentionTags.item(i).dataset.mentionTag;
                mentionTags.item(i).setAttribute('href', this.getTagUrl(value));
            }
        }
        // convert back to string
        return (new XMLSerializer()).serializeToString(dom);
    }
}
NgSimpleMarkdownViewerDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngSimpleMarkdownViewer]',
            },] }
];
NgSimpleMarkdownViewerDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: NgSimpleMarkdownService },
    { type: DomSanitizer }
];
NgSimpleMarkdownViewerDirective.propDecorators = {
    markdown: [{ type: Input }],
    styles: [{ type: Input }],
    getTagUrl: [{ type: Input }],
    mentions: [{ type: Output }],
    mentionWithIds: [{ type: Output }],
    ready: [{ type: Output }],
    tags: [{ type: Output }],
    taskLists: [{ type: Output }],
    border: [{ type: HostBinding, args: ['style.border',] }],
    iframeHeight: [{ type: HostBinding, args: ['style.height',] }],
    iframeWidth: [{ type: HostBinding, args: ['style.width',] }],
    onLoad: [{ type: HostListener, args: ['load',] }]
};

class NgSimpleMarkdownEditorModule {
}
NgSimpleMarkdownEditorModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    NgSimpleMarkdownEditorComponent,
                    NgSimpleMarkdownMentionDirective,
                    NgSimpleMarkdownMentionsComponent,
                    NgSimpleMarkdownTextPreviewPipe,
                    NgSimpleMarkdownViewerDirective,
                ],
                exports: [
                    NgSimpleMarkdownEditorComponent,
                    NgSimpleMarkdownTextPreviewPipe,
                    NgSimpleMarkdownViewerDirective,
                ],
                imports: [
                    CommonModule,
                    FormsModule,
                    MatButtonModule,
                    MatIconModule,
                    MatListModule,
                    ReactiveFormsModule,
                    TextFieldModule,
                ],
                providers: [
                    NgSimpleMarkdownService,
                ],
            },] }
];

/*
 * Public API Surface of ng-simple-markdown-editor
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgSimpleMarkdownEditorComponent, NgSimpleMarkdownEditorModule, NgSimpleMarkdownService, NgSimpleMarkdownTextPreviewPipe, NgSimpleMarkdownViewerDirective, NgSimpleMarkdownMentionDirective as ɵa, NgSimpleMarkdownMentionsComponent as ɵb };
//# sourceMappingURL=ng-simple-markdown-editor.js.map
